import z from 'zod';
import React, { ReactNode } from 'react';
import * as zod_to_json_schema from 'zod-to-json-schema';
import OpenAI from 'openai';

type ValuesOf<T> = T[keyof T];
type FilterObjectByValue<T, U> = {
    [K in keyof T]: T[K] extends U ? K : never;
};

/**
 * The type of authentication.
 */
declare const AuthType: {
    readonly NONE: "None";
    readonly TOKEN: "Token";
    readonly OAUTH: "OAuth";
};
type TAuthType = ValuesOf<typeof AuthType>;
type TAuthInputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z.AnyZodObject ? z.input<U> : {} : {};
type TAuthOutputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z.AnyZodObject ? z.output<U> : {} : {};
declare const OAuthConfigSchema: z.ZodUnion<[z.ZodObject<{
    scopes: z.ZodArray<z.ZodString, "many">;
    discoveryEndpoint: z.ZodString;
}, "strip", z.ZodTypeAny, {
    scopes: string[];
    discoveryEndpoint: string;
}, {
    scopes: string[];
    discoveryEndpoint: string;
}>, z.ZodObject<{
    scopes: z.ZodArray<z.ZodString, "many">;
    discoveryEndpoint: z.ZodUndefined;
    authorizationEndpoint: z.ZodString;
    tokenEndpoint: z.ZodString;
    refreshEndpoint: z.ZodOptional<z.ZodString>;
    revokeEndpoint: z.ZodOptional<z.ZodString>;
    codeChallengeMethod: z.ZodNullable<z.ZodLiteral<"S256">>;
}, "strip", z.ZodTypeAny, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}>]>;
type TOAuthConfig = z.infer<typeof OAuthConfigSchema>;
type TOAuthConfigWithOutputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthOutputMetadata<TRegistry, "oAuthMetadataSchema">;
type TOAuthConfigWithInputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthInputMetadata<TRegistry, "oAuthMetadataSchema">;
type TTokenCustomData = z.ZodObject<any> | null;
/**
 * Configuration for authenticating with an access token that users must generate themselves.
 *
 * This is useful for resources that don't support OAuth, but do support API keys.
 */
interface TTokenAuthConfig<TCustomData extends TTokenCustomData> {
    /**
     * Optional schema for custom data to be stored in the user's account that is needed to use the action.
     *
     * @example
     * For the Canvas LTS action, we also need to collect the user's Canvas domain (e.g. `https://canvas.calpoly.edu`).
     * ```
     * z.object({
     *  canvasDomain: z.string().url().refine((url) => url.startsWith("https://"), {
     *   message: "Canvas domain must start with https://",
     *  }),
     * })
     * ```
     */
    customDataSchema: TCustomData;
    /**
     * An optional function to validate the access token and custom data upon submission.
     *
     * @example
     * For the Canvas LTS action, we need to validate the access token and custom data by making a request to the Canvas API.
     *
     * ```typescript
     * validateToken: async ({ auth }) => {
     *  const response = await fetch(`${auth.customData.canvasDomain}/api/v1/users/self`, {
     *   headers: {
     *    Authorization: `Bearer ${auth.accessToken}`,
     *   },
     *  })
     *
     *  return { isValid: response.ok }
     * }
     * ```
     */
    validateToken?: (_: {
        auth: {
            customData: TCustomData extends z.ZodObject<any> ? z.infer<TCustomData> : null;
            accessToken: string;
        };
    }) => Promise<{
        isValid: boolean;
    }>;
}
type TTokenAuthConfigWithInputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthInputMetadata<TRegistry, "tokenAuthMetadataSchema">;
type TTokenAuthConfigWithOutputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthOutputMetadata<TRegistry, "tokenAuthMetadataSchema">;
interface TNoAuth {
    type: typeof AuthType.NONE;
    config: undefined;
}
interface TTokenAuth<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> {
    type: typeof AuthType.TOKEN;
    config: TTokenAuthConfigWithOutputMetadata<TCustomData, TRegistry>;
}
type TOAuth<TRegistry extends TAnyRegistryData> = {
    type: typeof AuthType.OAUTH;
    config: TOAuthConfigWithOutputMetadata<TRegistry>;
};
/**
 * The authentication configuration for an action.
 */
type TActionAuth<TRegistry extends TAnyRegistryData, TCustomData extends TTokenCustomData> = TNoAuth | TTokenAuth<TCustomData, TRegistry> | TOAuth<TRegistry>;
type TAnyActionAuth = TActionAuth<TAnyRegistryData, TTokenCustomData>;
/**
 * Generic type for getting the auth argument in the action input function.
 *
 * The auth argument will always have an `accessToken` property, but it may also have a `customData` property if the auth config has a custom data schema.
 */
type TAuthArg<TAuth extends TAnyActionAuth> = TAuth["type"] extends typeof AuthType.NONE ? never : TAuth["type"] extends typeof AuthType.OAUTH ? {
    accessToken: string;
} : TAuth["type"] extends typeof AuthType.TOKEN ? TAuth["config"] extends TTokenAuthConfigWithOutputMetadata<infer TCustomData, TAnyRegistryData> ? TCustomData extends TTokenCustomData ? {
    customData: TCustomData extends z.AnyZodObject ? z.output<TCustomData> : null;
    accessToken: string;
} : never : never : never;

type TActionMetadata = z.ZodType<any> | undefined;
type THandlerContext = z.ZodType<any> | undefined;
type TActionInput = z.ZodObject<any>;
type TActionOutput = z.ZodType<any>;
type TOptionalActionOutput = TActionOutput | undefined;
type TTokenAuthMetadata = z.ZodObject<any> | undefined;
type TOAuthMetadata = z.ZodObject<any> | undefined;
type TAdditionalParams = z.ZodType<any>;
type TOptionalAdditionalParams = TAdditionalParams | undefined;
type TStreamable = ReactNode | Promise<ReactNode>;
type TRenderReturn = TStreamable | Generator<TStreamable, TStreamable, void> | AsyncGenerator<TStreamable, TStreamable, void>;
interface TActionHandler<TRegistry extends TAnyRegistryData, TInput extends TActionInput, THandlerRet extends any, TAuth extends TAnyActionAuth, TAdditional extends TOptionalAdditionalParams> {
    (_: {
        input: z.output<TInput>;
    } & (TAuth["type"] extends "None" ? {} : {
        auth: TAuthArg<TAuth>;
    }) & (TAdditional extends TAdditionalParams ? {
        additionalParams: z.output<TAdditional>;
    } : {}) & (TRegistry["handlerContextSchema"] extends z.ZodType<any> ? {
        context: z.output<TRegistry["handlerContextSchema"]>;
    } : {})): THandlerRet;
}
interface TRegistryData<TNamespace extends string, TMetadata extends TActionMetadata, TContext extends THandlerContext, TToken extends TTokenAuthMetadata, TOAuth extends TOAuthMetadata> {
    /**
     * The namespace for the actions registry. Useful if you have multiple action registries.
     *
     * @example "ADE"
     */
    namespace: TNamespace;
    /**
     * The schema for the metadata of the action. This is used to validate the metadata passed to the action.
     *
     * If you don't have metadata, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *   title: z.string(),
     *   description: z.string(),
     * })
     * ```
     */
    metadataSchema?: TMetadata;
    /**
     * Sometimes you may want to pass extra data to each action handler (on top of input and auth). This is useful for things like user data, local time zone, etc.
     *
     * If you don't have any extra data, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *  userData: z.object({
     *   email: z.string().email(),
     *   name: z.string(),
     *  })
     * })
     * ```
     */
    handlerContextSchema?: TContext;
    /**
     * The schema for the token auth metadata. Useful if you want to store some extra data with the token.
     *
     * ```typescript
     * z.object({
     *  tokenGenerationDocumentationURL: z.string().url(),
     * })
     */
    tokenAuthMetadataSchema?: TToken;
    /**
     * The schema for the OAuth metadata. Useful if you want to store some extra data with the OAuth token.
     *
     * ```typescript
     * z.object({
     *   oauthAppGenerationDocumentationURL: z.string().url(),
     * })
     */
    oAuthMetadataSchema?: TOAuth;
}
interface TAnyRegistryData extends TRegistryData<string, TActionMetadata, THandlerContext, TTokenAuthMetadata, TOAuthMetadata> {
}
interface TActionData<TRegistry extends TAnyRegistryData, TId extends string, TFunctionName extends string, TInput extends TActionInput, TOutput extends TOptionalActionOutput, TAdditional extends TOptionalAdditionalParams, TAuth extends TAnyActionAuth, THandlerRet extends any> {
    registryData: TRegistry;
    id: TId;
    functionName: TFunctionName;
    metadata: TRegistry["metadataSchema"] extends z.ZodType<any> ? z.output<TRegistry["metadataSchema"]> : undefined;
    inputSchema: TInput;
    outputSchema: TOutput;
    additionalParamsSchema: TAdditional;
    authConfig: TAuth;
    exampleInput: z.input<TInput> | null;
    handler: TActionHandler<TRegistry, TInput, THandlerRet, TAuth, TAdditional>;
    $unwrappedHandler: TActionHandler<TRegistry, TInput, THandlerRet, TAuth, TAdditional>;
    render: ((props: Parameters<TActionHandler<TRegistry, TInput, any, TAuth, TAdditional>>[0] & {
        handler: TActionHandler<TRegistry, TInput, THandlerRet, TAuth, TAdditional>;
    }) => TRenderReturn) | undefined;
}
interface TAnyActionData extends TActionData<any, any, any, any, any, any, any, any> {
}

declare class ActionBuilderWithHandler<TLocalActionData extends TAnyActionData> {
    id: TLocalActionData["id"];
    functionName: TLocalActionData["functionName"];
    metadata: TLocalActionData["metadata"];
    registryData: TLocalActionData["registryData"];
    exampleInput: z.input<TLocalActionData["inputSchema"]>;
    inputSchema: TLocalActionData["inputSchema"];
    outputSchema: TLocalActionData["outputSchema"];
    additionalParamsSchema: TLocalActionData["additionalParamsSchema"];
    auth: TLocalActionData["authConfig"];
    _render: TLocalActionData["render"];
    handler: TLocalActionData["handler"];
    $unwrappedHandler: TLocalActionData["handler"];
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setExampleInput(exampleInput: z.input<TLocalActionData["inputSchema"]>): this;
    render(renderFunction: TLocalActionData["render"]): this;
    getInputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getOutputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getChatCompletionTool(): OpenAI.Chat.Completions.ChatCompletionTool;
}

declare class ActionBuilderWithOAuthType<TLocalActionData extends TActionDataWithInput, TOutput extends TOptionalActionOutput, TAdditional extends TOptionalAdditionalParams> {
    _actionData: TLocalActionData;
    _outputSchema: TOutput;
    _additionalParamsSchema: TAdditional;
    constructor({ actionData, outputSchema, additionalParamsSchema, }: {
        actionData: TLocalActionData;
        outputSchema: TOutput;
        additionalParamsSchema: TAdditional;
    });
    oAuthData: (data: TOAuthConfigWithInputMetadata<TLocalActionData["registryData"]>) => Omit<ActionBuilderWithInput<TLocalActionData, TOutput, TAdditional, {
        type: "OAuth";
        config: TOAuthConfigWithOutputMetadata<TLocalActionData["registryData"]>;
    }>, TOmitOnInputWithAuth<TOutput>>;
}

declare class ActionBuilderWithTokenType<TLocalActionData extends TActionDataWithInput, TOutput extends TOptionalActionOutput, TAdditional extends TOptionalAdditionalParams> {
    _actionData: TLocalActionData;
    _outputSchema: TOutput;
    _additionalParamsSchema: TAdditional;
    constructor({ actionData, outputSchema, additionalParamsSchema, }: {
        actionData: TLocalActionData;
        outputSchema: TOutput;
        additionalParamsSchema: TAdditional;
    });
    tokenData: <T extends TTokenCustomData>(data: TTokenAuthConfigWithInputMetadata<T, TLocalActionData["registryData"]>) => Omit<ActionBuilderWithInput<TLocalActionData, TOutput, TAdditional, TTokenAuth<T, TLocalActionData["registryData"]>>, TOmitOnInputWithAuth<TOutput>>;
}

type TActionBuilderWithInputData<TBuilderData extends TActionBuilderData, TInput extends TActionInput> = TBuilderData & Pick<TActionData<any, any, any, TInput, any, any, any, any>, "inputSchema">;
interface TActionDataWithInput extends TActionBuilderWithInputData<TActionBuilderData, TActionInput> {
}
type TOmitOnInputBase = "_actionData" | "_outputSchema" | "_authConfig" | "_additionalParamsSchema";
type TOmitOnInputWithOutput = TOmitOnInputBase | "output" | "outputSameAsInput" | "additionalParams";
type TOmitOnInputWithAdditionalParams = TOmitOnInputBase | "additionalParams";
type TOmitOnInputWithAuth<TOutput extends TOptionalActionOutput> = TOmitOnInputWithOutput | TOmitOnInputWithAdditionalParams | (TOutput extends undefined ? "setAuthType" : "setAuthType" | "noHandler");
declare class ActionBuilderWithInput<TLocalActionData extends TActionDataWithInput, TOutput extends TOptionalActionOutput, TAdditional extends TOptionalAdditionalParams, TAuth extends TActionAuth<TLocalActionData["registryData"], TTokenCustomData>> {
    _actionData: TLocalActionData;
    _outputSchema: TOutput;
    _authConfig: TAuth;
    _additionalParamsSchema: TAdditional;
    constructor({ actionData, outputSchema, authConfig, additionalParamsSchema, }: {
        actionData: TLocalActionData;
        outputSchema: TOutput;
        authConfig: TAuth;
        additionalParamsSchema: TAdditional;
    });
    /**
     * `JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     */
    output<T extends TActionOutput | z.ZodRawShape>(output: T): Omit<ActionBuilderWithInput<TLocalActionData, T extends z.ZodRawShape ? z.ZodObject<T> : T, TAdditional, TAuth>, TOmitOnInputWithOutput | "noHandler">;
    /**
     * Sets the output schema to be the same as the input schema.
     */
    outputSameAsInput(): Omit<ActionBuilderWithInput<TLocalActionData, TLocalActionData["inputSchema"], TAdditional, TAuth>, TOmitOnInputWithOutput>;
    /**
     * The additional parameters for the action. This is useful to pass in additional data that the action needs to run.
     *
     * @param additionalParams The additional parameters for the action.
     * @returns The action builder with the additional parameters set.
     * @example
     */
    additionalParams<T extends TAdditionalParams | z.ZodRawShape>(additionalParams: T): Omit<ActionBuilderWithInput<TLocalActionData, TOutput, T extends z.ZodRawShape ? z.ZodObject<T> : T, TAuth>, TOmitOnInputWithAdditionalParams>;
    /**
     * The authentication type for the action. This is used to generate the authentication page
     * for the action and get users authenticated.
     */
    authType(type: typeof AuthType.TOKEN): Omit<ActionBuilderWithTokenType<TLocalActionData, TOutput, TAdditional>, "_actionData" | "_outputSchema">;
    authType(type: typeof AuthType.OAUTH): Omit<ActionBuilderWithOAuthType<TLocalActionData, TOutput, TAdditional>, "_actionData" | "_outputSchema">;
    /**
     * The handler that gets called when the action is invoked. This handler should return a promise that resolves to the output of the action.
     *
     * @param input The input to the action handler. This is the input that the user provides when invoking the action.
     * @param auth The authentication information for the action. This is the authentication information that the user provides when invoking the action.
     *
     * @returns A promise that resolves to the output of the action.
     *
     * @example
     * The following action handler is an example for a `send-email` action.
     * ```
     * async ({ input, auth }) => {
     *   const { subject, body, to } = input;
     *   const { accessToken } = auth;
     *   ...code to send email using the input...
     *   return;
     * }
     * ```
     *
     * @example
     * The following action handler is an example for a `get-google-contact` action.
     * ```
     * async ({ input, auth }) => {
     *   const contact = ...code to get contact...
     *   return contact;
     * }
     * ```
     */
    handler<THandlerRet extends TOutput extends infer TOutput ? TOutput extends TActionOutput ? Promise<z.input<TOutput>> | z.input<TOutput> : Promise<any> | any : Promise<any> | any>(handler: TActionHandler<TLocalActionData["registryData"], TLocalActionData["inputSchema"], THandlerRet, TAuth, TAdditional>): ActionBuilderWithHandler<TActionData<TLocalActionData["registryData"], TLocalActionData["id"], TLocalActionData["functionName"], TLocalActionData["inputSchema"], TOutput, TAdditional, TAuth, TOutput extends TActionOutput ? THandlerRet extends Promise<z.input<TOutput>> ? Promise<z.output<TOutput>> : z.output<TOutput> : THandlerRet>>;
    noHandler(): ActionBuilderWithHandler<TActionData<TLocalActionData["registryData"], TLocalActionData["id"], TLocalActionData["functionName"], TLocalActionData["inputSchema"], TLocalActionData["inputSchema"], TAdditional, TAuth, z.output<TLocalActionData["inputSchema"]>>>;
}

interface TActionBuilderConstructorData<TRegistry extends TAnyRegistryData, TId extends string, TFunctionName extends string> {
    id: TId;
    functionName: TFunctionName;
    metadata: TRegistry["metadataSchema"] extends z.ZodType<any> ? z.output<TRegistry["metadataSchema"]> : undefined;
    registryData: TRegistry;
}
interface TActionBuilderData extends TActionBuilderConstructorData<TAnyRegistryData, string, string> {
}
type TOmitActionBuilderBase = "_actionData" | "_description";
declare class ActionBuilder<TLocalActionData extends TActionBuilderData> {
    _actionData: TLocalActionData;
    _description: string | undefined;
    constructor(input: TLocalActionData, description?: string);
    /**
     * The description of the action. This is used to generate the prompts and details page for your action.
     * @param description The description of the action.
     * @returns The action builder with the description set.
     * @example
     */
    describe(description: string): Omit<this, "describe" | TOmitActionBuilderBase>;
    /**
     * The Zod input schema for the action. This is used to validate the arguments passed to the action.
     *
     * You must include a `.describe()` call on each field to provide a description of the input. This description will be used to generate the prompts and details page for your action.
     *
     * @example
     * The following input schema is an example for a `send-email` action.
     * ```
     * z.object({
     *  subject: z.string().desribe("The subject of the email"),
     *  body: z.string().describe("The body of the email"),
     *  to: z.array(z.object({
     *   email: z.string().describe("The email of the recipient"),
     * })).describe("The recipients of the email"),
     * ```
     */
    input<T extends z.ZodRawShape>(input: T): Omit<ActionBuilderWithInput<TLocalActionData & {
        inputSchema: z.ZodObject<T, "strip", z.ZodTypeAny, { [k_1 in keyof z.objectUtil.addQuestionMarks<z.baseObjectOutputType<T>, { [k in keyof z.baseObjectOutputType<T>]: undefined extends z.baseObjectOutputType<T>[k] ? never : k; }[keyof T]>]: z.objectUtil.addQuestionMarks<z.baseObjectOutputType<T>, { [k_2 in keyof z.baseObjectOutputType<T>]: undefined extends z.baseObjectOutputType<T>[k_2] ? never : k_2; }[keyof T]>[k_1]; }, { [k_2 in keyof z.baseObjectInputType<T>]: z.baseObjectInputType<T>[k_2]; }>;
    }, undefined, undefined, {
        type: "None";
        config: undefined;
    }>, TOmitOnInputBase>;
}

type inferNamespace$1<T extends TAnyClientActionRegistry> = T[keyof T]["namespace"] extends string ? `${T[keyof T]["namespace"]}ActionsRegistry` : never;
type TBase = {
    [K in string]: TAnyClientActionRegistry;
};
type TActionRegistriesContext<TMap extends TBase = TBase> = {
    serializedRegistryMap: TMap;
} | undefined;
declare function createUseActionRegistries<const TContext extends NonNullable<TActionRegistriesContext>>(useContext: () => TActionRegistriesContext): () => TContext["serializedRegistryMap"];
declare function prepareAIActions<TRegistries extends Array<TAnyClientActionRegistry>>(actionRegistries: TRegistries): NonNullable<TActionRegistriesContext<{
    [K in TRegistries[number] as inferNamespace$1<K>]: K;
}>>;

type TClientMetadata = z.output<z.ZodType<any>> | undefined;
interface ClientActionData<TNamespace extends string, TId extends string, TFunctionName extends string, TInputSchema extends TActionInput, TOutputSchema extends TOptionalActionOutput, TMetadata extends TClientMetadata> {
    namespace: TNamespace;
    actionId: TId;
    functionName: TFunctionName;
    inputSchema: TInputSchema;
    outputSchema: TOutputSchema;
    metadata: TMetadata;
}
type TAnyClientActionData = ClientActionData<any, any, any, any, any, any>;
interface TAnyClientActionRegistry extends Readonly<{
    [K in string]: TAnyClientActionData;
}> {
}
type inferActionComponentPropsBase<T extends TAnyClientActionData> = {
    args: z.output<T["inputSchema"]> | null;
    functionName: T["functionName"];
    inputSchema: T["inputSchema"];
    outputSchema: T["outputSchema"];
    onSubmit: (args: z.output<T["inputSchema"]>) => void;
} & (T["metadata"] extends z.output<any> ? {
    metadata: T["metadata"];
} : {});
type inferActionComponentRouter<T extends TAnyClientActionRegistry, Props extends {} = {}> = {
    [K in keyof T as T[K]["functionName"]]: Props & inferActionComponentPropsBase<T[K]>;
};
type inferActionComponentProps<T extends inferActionComponentRouter<TAnyClientActionRegistry, any>, K extends keyof T> = T[K];
declare const createActionComponentRouter: <TRouter extends inferActionComponentRouter<TAnyClientActionRegistry, any>>(args: { [K in keyof TRouter]: React.FC<inferActionComponentProps<TRouter, K>>; }) => React.FC<(TRouter extends inferActionComponentRouter<any, infer K_1 extends {}> ? K_1 : {
    other: string;
}) & {
    functionName: string;
    args: unknown;
    inputSchema: TRouter[keyof TRouter]["inputSchema"];
    outputSchema: TRouter[keyof TRouter]["outputSchema"];
    onSubmit: (args: ValuesOf<{ [K_2 in keyof TRouter]: {
        functionName: K_2;
        args: NonNullable<TRouter[K_2]["args"]>;
    }; }>) => void;
    fallback?: React.ReactNode;
    metadata: TRouter[keyof TRouter]["metadata"];
}>;
type inferNamespace<T extends TAnyActionRegistry> = T[keyof T]["registryData"]["namespace"];
type inferClientReturn<T extends TAnyActionRegistry, TTransformedMetadata extends TClientMetadata> = {
    [K in keyof T]: ClientActionData<inferNamespace<T>, T[K]["id"], T[K]["functionName"], T[K]["inputSchema"], T[K]["outputSchema"], TTransformedMetadata>;
};
type TCreateClientActionsRegistryOptions<T extends TAnyActionRegistry, TTransformedMetadata extends TClientMetadata = undefined> = T[keyof T]["metadata"] extends undefined ? {} : {
    pipeMetadata?: (metadata: T[keyof T]["metadata"]) => TTransformedMetadata;
};
declare const createClientActionsRegistry: <const T extends TAnyActionRegistry, TTransformedMetadata extends unknown = undefined>(registry: T, options?: TCreateClientActionsRegistryOptions<T, TTransformedMetadata> | undefined) => inferClientReturn<T, TTransformedMetadata>;

type TAnyActionRegistry$1 = Readonly<{
    [key: string]: ActionBuilderWithHandler<any>;
}>;
type TActionCallerInput<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: {
        name: TRegistry[K]["functionName"];
        arguments: z.input<TRegistry[K]["inputSchema"]>;
        id?: string;
    };
}>;
interface TCallerSuccessResult<TAction extends ActionBuilderWithHandler<any>> {
    status: "success";
    data: Awaited<ReturnType<TAction["handler"]>>;
    actionId: TAction["id"];
    functionName: TAction["functionName"];
    id: string;
    arguments: z.input<TAction["inputSchema"]>;
    parsedArguments: z.output<TAction["inputSchema"]>;
    additionalParams: TAction["additionalParamsSchema"] extends undefined ? undefined : z.input<TAction["additionalParamsSchema"]>;
    parsedAdditionalParams: TAction["additionalParamsSchema"] extends undefined ? undefined : z.output<TAction["additionalParamsSchema"]>;
    isSuccess: true;
    isError: false;
    $parsedContext?: any;
    $auth?: any;
}
interface TCallerErrorResult<TAction extends ActionBuilderWithHandler<any>> {
    status: "error";
    message: string;
    cause?: Error;
    actionId: TAction["id"];
    data: undefined;
    isError: true;
    isSuccess: false;
    id: string;
    failedArguments: z.input<TAction["inputSchema"]>;
    functionName: TAction["functionName"];
    failedParsedArguments: z.output<TAction["inputSchema"]> | null;
}
type TActionRegistrySubset<TRegistry extends TAnyActionRegistry$1, TSubset extends (keyof TRegistry)[] | undefined = undefined> = TSubset extends (keyof TRegistry)[] ? TSubset[number] : keyof TRegistry;
type TCallerResults<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: TCallerSuccessResult<TRegistry[K]> | TCallerErrorResult<TRegistry[K]>;
}>[];
type inferContext<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = TRegistry[TId]["registryData"] extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["handlerContextSchema"] extends z.ZodType<any> ? {
    context: z.input<TRegistry["handlerContextSchema"]>;
} : {} : {} : {};
type inferAdditionalParamKeys<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = {
    [K in TId as TRegistry[K]["additionalParamsSchema"] extends TAdditionalParams ? K : never]: z.input<TRegistry[K]["additionalParamsSchema"]>;
};
type inferAdditionalParams<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = inferAdditionalParamKeys<TRegistry, TId> extends infer TKeys ? keyof TKeys extends never ? {} : {
    additionalParams: {
        [K in keyof TKeys]: TKeys[K];
    };
} : {};
type filterByAuthType<T extends TAnyActionRegistry$1, U extends TAuthType, TId extends keyof T, TSuccess> = {
    [K in TId as T[K]["auth"]["type"] extends U ? K : never]: TSuccess;
};
/**
 * A function that fetches the OAuth access token for an action.
 *
 * @param actionId The ID of the action to fetch the OAuth access token for.
 * @returns A promise that resolves to the OAuth access token as { accessToken: string }.
 *
 * @throws An error if the OAuth access token could not be fetched.
 *
 * @example
 * ```ts
 * async fetchOAuthAccessToken(actionId) {
 *    ...fetch the OAuth access token...
 *    return {
 *        accessToken: "the access token"
 *    }
 * }
 *
 */
interface TFetchOAuthAccessToken<T> {
    (actionId: T): Promise<{
        accessToken: string;
    }> | {
        accessToken: string;
    };
}
type inferOAuthFunction<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "OAuth", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchOAuthAccessToken: TFetchOAuthAccessToken<keyof A>;
} : {};
interface TFetchTokenAuthData<T> {
    (actionId: T): Promise<{
        accessToken: string;
        customData: z.AnyZodObject["_input"];
    }> | {
        accessToken: string;
        customData: z.AnyZodObject["_input"];
    };
}
type inferTokenFunction<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "Token", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchTokenAuthData: TFetchTokenAuthData<keyof A>;
} : {};
interface TActionStarted<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> {
    (result: Pick<TCallerSuccessResult<TRegistry[TId]>, "actionId" | "arguments" | "functionName" | "id"> & {
        timestamp: number;
    }): void;
}
interface TActionFinished<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> {
    (result: TCallerResults<TRegistry, TId>[number] & {
        timestamp: number;
    }): void;
}
type TDynamicFunctionCallingArgs<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = inferOAuthFunction<TRegistry, TId> & inferTokenFunction<TRegistry, TId> & inferContext<TRegistry, TId> & inferAdditionalParams<TRegistry, TId>;
type TFunctionCallingArgs<TRegistry extends TAnyActionRegistry$1, U extends (keyof TRegistry)[] | undefined = undefined> = {
    inArray?: U;
    runInParallel?: boolean;
    mode?: "render" | "handler";
    onActionExecutionStarted?: TActionStarted<TRegistry, TActionRegistrySubset<TRegistry, U>>;
    onActionExecutionFinished?: TActionFinished<TRegistry, TActionRegistrySubset<TRegistry, U>>;
} & TDynamicFunctionCallingArgs<TRegistry, TActionRegistrySubset<TRegistry, U>>;
interface TActionCaller<TActionData extends TAnyActionData, TRegistry extends Readonly<{
    [K in TActionData["id"]]: ActionBuilderWithHandler<TActionData>;
}>, U extends (keyof TRegistry)[] | undefined = undefined> {
    (inputs: TActionCallerInput<TRegistry, TActionRegistrySubset<TRegistry, U>>[]): Promise<TCallerResults<TRegistry, TActionRegistrySubset<TRegistry, U>>>;
}
declare const setupActionCaller: <TActionData extends TAnyActionData, const TRegistry extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithHandler<TActionData>; }>, U extends (keyof TRegistry)[] | undefined = undefined>(registry: TRegistry, args: TFunctionCallingArgs<TRegistry, U>) => {
    /**
     * A function that calls the actions.
     *
     * Returns the results of the actions. Results are in the same order as the inputs and contain the status of the action, the data if the action was successful, and the action ID.
     *
     * @param inputs The inputs to the actions.
     * @returns The results of the actions.
     */
    actionCaller: TActionCaller<TActionData, TRegistry, U>;
};

/**
 * Given an action registry, generate the functions for the LLM.
 */
declare const generateLLMFunctions: <const T extends Readonly<{
    [key: string]: ActionBuilderWithHandler<any>;
}>, U extends (keyof T)[] | undefined>(registry: T, args?: {
    inArray?: U | undefined;
} | undefined) => OpenAI.Chat.Completions.ChatCompletionCreateParams.Function[];
interface TFunctionCallHandler<T extends TAnyActionRegistry, U extends (keyof T)[] | undefined> {
    (functionCall: {
        name: string;
        arguments: Record<string, unknown> | string;
    }): Promise<{
        functionCallMessage: OpenAI.Chat.Completions.ChatCompletionFunctionMessageParam;
        results: TCallerResults<T, TActionRegistrySubset<T, U>>;
    }>;
}
interface TCreateFewShotFunctionCallMessages<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined> {
    (examples: {
        userMessageContent: string;
        assistantMessageContent?: string;
        function_call: TFewShotExampleCalls<TRegistry, U>;
    }[]): OpenAI.Chat.Completions.ChatCompletionMessageParam[];
}
interface TChooseFunction<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined = undefined> {
    (name: TRegistry[TActionRegistrySubset<TRegistry, U>]["functionName"]): OpenAI.Chat.Completions.ChatCompletionFunctionCallOption;
}
declare const setupFunctionCalling: <TActionData extends TAnyActionData, const T extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithHandler<TActionData>; }>, U extends (keyof T)[] | undefined = undefined>(registry: T, args: TFunctionCallingArgs<T, U>) => {
    functions: OpenAI.Chat.Completions.ChatCompletionCreateParams.Function[];
    chooseFunction: TChooseFunction<T, U>;
    createFewShotFunctionCallMessages: TCreateFewShotFunctionCallMessages<T, U>;
    functionCallHandler: TFunctionCallHandler<T, U>;
};

/**
 * Filter the action registry by the auth type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByAuthType: <const T extends TAnyActionRegistry, U extends TAuthType>(registry: T, authType: U) => { [K in keyof T as T[K]["auth"]["type"] extends U ? K : never]: T[K]; };
/**
 * Filter the action registry by metadat
 *
 * @param registry The registry of actions.
 * @param filterFn The filter function to apply to the actions.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByMetadata: <const T extends TAnyActionRegistry>(registry: T, fn: (metadata: T[keyof T]["metadata"]) => boolean) => T;
/**
 * Create a readonly map of {
 *   [id]: id
 * }
 */
declare const generateActionIdMap: <const T extends TAnyActionRegistry>(registry: T) => Readonly<{ [K in keyof T]: K; }>;
/**
 * Pick the actions from the registry.
 *
 * @param registry The registry of actions.
 * @param id The id of the action to pick.
 *
 * @returns The picked actions.
 *
 */
declare const pickFromActionsRegistry: <T extends TAnyActionRegistry, K extends keyof T | (keyof T)[]>(registry: T, id: K) => K extends infer U ? U extends (keyof T)[] ? Pick<T, U[number]> : U extends keyof T ? Pick<T, U> : never : never;
declare const omitFromActionsRegistry: <T extends TAnyActionRegistry, K extends keyof T | (keyof T)[]>(registry: T, id: K) => K extends infer U ? U extends (keyof T)[] ? Omit<T, U[number]> : U extends keyof T ? Omit<T, U> : never : never;

interface TActionsArray<TRegistryData extends TAnyRegistryData> extends Array<ReturnType<ActionBuilderWithHandler<any>["registryData"]> extends TRegistryData ? ActionBuilderWithHandler<any> : never> {
}
type inferId<T extends TAnyActionRegistry[string]> = T["id"];
type TActionsRegistry<TRegistryData extends TAnyRegistryData, T extends TActionsArray<TRegistryData>> = {
    [K in T[number] as K["registryData"] extends TRegistryData ? inferId<K> : never]: K;
};
interface TAnyActionRegistry extends Readonly<{
    [key: string]: ActionBuilderWithHandler<any>;
}> {
}
interface TCreateActionsRegistryFunction<TRegistryData extends TAnyRegistryData> {
    <const T extends TActionsArray<TRegistryData>>(registry: T): TActionsRegistry<TRegistryData, T>;
}
interface TCreateActionFunction<TRegistryData extends TAnyRegistryData> {
    <TId extends string, TFunctionName extends string = TId>(input: {
        id: TId;
        functionName?: TFunctionName;
    } & (TRegistryData["metadataSchema"] extends z.ZodType<any> ? {
        metadata: z.input<TRegistryData["metadataSchema"]>;
    } : {})): Omit<ActionBuilder<TActionBuilderConstructorData<TRegistryData, TId, TFunctionName>>, "_actionData" | "_description">;
}
/**
 * Generate the functions to create an action registry.
 *
 * @param args The arguments to generate the functions.
 * @returns The functions to create an action registry.
 *
 * @example
 * ```
 * const { createADEActionsRegistry, createADEAction } = generateActionRegistryFunctions({
 *   namespace: "ADE",
 *   metadataSchema: z.object({
 *     title: z.string(),
 *     description: z.string(),
 *   }),
 *   handlerContextSchema: z.object({
 *     userData: z.object({
 *       email: z.string().email(),
 *       name: z.string(),
 *     })
 *   }),
 * })
 * ```
 */
declare const generateActionRegistryFunctions: <const TRegistryData extends TAnyRegistryData>(args: TRegistryData) => Readonly<{ [K in `create${TRegistryData["namespace"]}ActionsRegistry`]: TCreateActionsRegistryFunction<TRegistryData>; } & { [K_1 in `create${TRegistryData["namespace"]}Action`]: TCreateActionFunction<TRegistryData>; }>;
declare const createActionsRegistry: TCreateActionsRegistryFunction<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly handlerContextSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}>;
declare const createAction: TCreateActionFunction<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly handlerContextSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}>;
type inferActionRegistryInputs<TRegistry extends TAnyActionRegistry> = {
    [K in keyof TRegistry]: z.output<TRegistry[K]["inputSchema"]>;
};
type inferActionRegistryOutputs<TRegistry extends TAnyActionRegistry> = {
    [K in keyof TRegistry]: Awaited<ReturnType<TRegistry[K]["handler"]>>;
};

/**
 * Given an action registry, generate the tools for the LLM.
 */
declare const generateLLMTools: <const T extends Readonly<{
    [key: string]: ActionBuilderWithHandler<any>;
}>, U extends (keyof T)[] | undefined>(registry: T, args?: {
    inArray?: U | undefined;
} | undefined) => OpenAI.Chat.Completions.ChatCompletionTool[];
interface TToolCallHandler<T extends TAnyActionRegistry, U extends (keyof T)[] | undefined> {
    (toolCalls: OpenAI.Chat.Completions.ChatCompletionMessageToolCall[]): Promise<{
        toolCallMessages: OpenAI.Chat.Completions.ChatCompletionToolMessageParam[];
        results: TCallerResults<T, TActionRegistrySubset<T, U>>;
    }>;
}
type TFewShotExampleCalls<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined> = ValuesOf<{
    [K in TActionRegistrySubset<TRegistry, U>]: {
        name: TRegistry[K]["functionName"];
        arguments: z.input<TRegistry[K]["inputSchema"]>;
    } & (Awaited<ReturnType<TRegistry[K]["handler"]>> extends void ? {} : {
        response: Awaited<ReturnType<TRegistry[K]["handler"]>>;
    });
}>;
interface TCreateFewShotToolCallMessages<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined> {
    (examples: {
        userMessageContent: string;
        assistantMessageContent?: string;
        tool_calls: TFewShotExampleCalls<TRegistry, U>[];
    }[]): OpenAI.Chat.Completions.ChatCompletionMessageParam[];
}
interface TChooseTool<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined = undefined> {
    (name: TRegistry[TActionRegistrySubset<TRegistry, U>]["functionName"]): OpenAI.Chat.Completions.ChatCompletionToolChoiceOption;
}
type TToolsWithRender<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined = undefined> = {
    [K in TActionRegistrySubset<TRegistry, U>]: {
        description?: string | undefined;
        parameters: TRegistry[K]["inputSchema"];
        render: (props: z.output<TRegistry[K]["inputSchema"]>) => AsyncGenerator<TStreamable, TStreamable, void>;
    };
};
declare const setupToolCalling: <TActionData extends TAnyActionData, const T extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithHandler<TActionData>; }>, U extends (keyof T)[] | undefined = undefined>(registry: T, args: {
    inArray?: U | undefined;
    runInParallel?: boolean | undefined;
    mode?: "render" | "handler" | undefined;
    onActionExecutionStarted?: TActionStarted<T, TActionRegistrySubset<T, U>> | undefined;
    onActionExecutionFinished?: TActionFinished<T, TActionRegistrySubset<T, U>> | undefined;
} & ((TActionRegistrySubset<T, U> extends infer T_1 extends keyof T ? { [K in T_1 as T[K]["auth"]["type"] extends "OAuth" ? K : never]: 1; } : never) extends infer A ? keyof A extends never ? {} : {
    fetchOAuthAccessToken: TFetchOAuthAccessToken<keyof A>;
} : {}) & ((TActionRegistrySubset<T, U> extends infer T_2 extends keyof T ? { [K_1 in T_2 as T[K_1]["auth"]["type"] extends "Token" ? K_1 : never]: 1; } : never) extends infer A_1 ? keyof A_1 extends never ? {} : {
    fetchTokenAuthData: TFetchTokenAuthData<keyof A_1>;
} : {}) & (T[TActionRegistrySubset<T, U>]["registryData"] extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["handlerContextSchema"] extends z.ZodType<any, z.ZodTypeDef, any> ? {
    context: z.input<TRegistry["handlerContextSchema"]>;
} : {} : {} : {}) & ((TActionRegistrySubset<T, U> extends infer T_3 extends keyof T ? { [K_2 in T_3 as T[K_2]["additionalParamsSchema"] extends TAdditionalParams ? K_2 : never]: z.input<T[K_2]["additionalParamsSchema"]>; } : never) extends infer TKeys ? keyof TKeys extends never ? {} : {
    additionalParams: { [K_3 in keyof TKeys]: TKeys[K_3]; };
} : {}) & {
    /**
     * Whether to include errors in the tool call messages.
     */
    includeErrorsInToolCallMessages?: boolean | undefined;
}) => {
    tools: OpenAI.Chat.Completions.ChatCompletionTool[];
    toolCallsHandler: TToolCallHandler<T, U>;
    chooseTool: TChooseTool<T, U>;
    createFewShotToolCallMessages: TCreateFewShotToolCallMessages<T, U>;
    toolsWithRender: TToolsWithRender<T, U>;
};

export { AuthType, type ClientActionData, type FilterObjectByValue, type TActionBuilderConstructorData, type TActionCallerInput, type TActionData, type TActionFinished, type TActionHandler as TActionFunction, type TActionInput, type TActionMetadata, type TActionRegistriesContext, type TActionRegistrySubset, type TActionStarted, type TActionsArray, type TAnyActionRegistry, type TAnyClientActionRegistry, type TCallerErrorResult, type TCallerResults, type TCallerSuccessResult, type TCreateActionFunction, type TCreateActionsRegistryFunction, type TDynamicFunctionCallingArgs, type TFetchOAuthAccessToken, type TFetchTokenAuthData, type TFewShotExampleCalls, type TFunctionCallingArgs, type TNoAuth, type TTokenAuth, type TTokenAuthConfig, type ValuesOf, createAction, createActionComponentRouter, createActionsRegistry, createClientActionsRegistry, createUseActionRegistries, filterActionRegistryByAuthType, filterActionRegistryByMetadata, generateActionIdMap, generateActionRegistryFunctions, generateLLMFunctions, generateLLMTools, type inferActionComponentProps, type inferActionComponentRouter, type inferActionRegistryInputs, type inferActionRegistryOutputs, omitFromActionsRegistry, pickFromActionsRegistry, prepareAIActions, setupActionCaller, setupFunctionCalling, setupToolCalling };
